==> bibledit-1.3.tar.gz: bibledit-1.3/src/shell.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "shell.h"
#include "gwrappers.h"
#include "directories.h"
#include "utilities.h"


bool program_is_running (const ustring & commandline)
{
  /* 
     Returns the number of programs given on "commandline"
     that are running.
   */
  // Run the process status program.
  FILE *fd;
  fd = popen ("ps ax", "r");
  if (!fd) {
    return false;
  }
  // Read the results of it.
  int i = 0;
  char buf[1024];
  while (fgets (buf, sizeof (buf), fd) != NULL) {
    if (strstr (buf, commandline.c_str ()) != NULL)
      i++;
  }
  int result;
  result = pclose (fd);
  // Return true if one or more of the commandlines were found.
  // That is, return true if the program runs.
  return (i > 0);
}


int run_shell_progress (const ustring& commandline, const ustring& programname, Progress * progress, bool output_to_logfile)
/*
Some shell processes take a long time.
This function runs that shell process, and updates the progressbar also,
so that the user has some feedback and sees things moving forward.
*/
{
  // Return value to indicate success or not.
  int returnvalue = -1;
  // Start progress.
  progress->start();
  // Make a logfile.
  ustring logfile;
  logfile = gw_build_filename (directories_get_temp(), "shell.log");
  // Start the process in the background.
  ustring command (commandline);
  if (output_to_logfile) {
    command.append (" > ");
    command.append (logfile);
    command.append (" 2>&1");
  }
  command.append (" &");
  if (system (command.c_str()) >= 0) {
    // Indicate zero for no error.
    returnvalue = 0;
    // We show a moving fraction.
    double fraction = 0;
    // Wait for the shell process to finish.
    while (program_is_running (programname)) {
      fraction += 0.05;
      if (fraction >= 1)
        fraction = 0;
      progress->set_fraction (fraction);
      if (progress->cancel) {
        // Kill formatter.
        ustring command;
        command = "killall -HUP ";
        command.append (programname);
        system (command.c_str());
        returnvalue = -1;
        break;
      }
      g_usleep (1000000);
    }
  }
  // Read the logfile made by the shell command.
  if (output_to_logfile) {
    ReadText rt (logfile, true, false);
    for (unsigned int i = 0; i < rt.lines.size(); i++) {
      // Add to program's logfile.
      gw_message (rt.lines[i]);
    }
  }
  // Finish.
  progress->finish();
  return returnvalue;  
}

==> bibledit-1.4.tar.gz: bibledit-1.4/src/shell.cpp <==
/*
** Copyright (C) 2003-2005 The Free Software Foundation.
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/


#include "libraries.h"
#include "shell.h"
#include "gwrappers.h"
#include "directories.h"
#include "utilities.h"
#include "gtkwrappers.h"


bool program_is_running (const ustring & commandline)
// Returns true if the program given on "commandline" is running.
{
  // Run the process status program.
  FILE *fd;
  fd = popen ("ps ax", "r");
  if (!fd) {
    return false;
  }
  // Read the results of it.
  int i = 0;
  char buf[1024];
  while (fgets (buf, sizeof (buf), fd) != NULL) {
    if (strstr (buf, commandline.c_str ()) != NULL)
      i++;
  }
  int result;
  result = pclose (fd);
  // Return true if one or more of the commandlines were found.
  // That is, return true if the program runs.
  return (i > 0);
}


int run_shell_progress (const ustring& commandline, const ustring& programname, 
                        Progress * progress, const ustring& message, 
                        bool output_to_logfile, int max_logfile_size)
/*
Some shell processes take a long time.
This function runs that shell process, and updates the progressbar also,
so that the user has some feedback and sees things moving forward.
The size of the logfile must not exceeded "max_logfile_size". If it does,
the process will be interrupted.
*/
{
  // Return value to indicate success or not.
  int returnvalue = -1;
  // Check logfile size.
  bool size_exceeded = false;
  // Start progress.
  progress->start();
  progress->set_text (message);
  // Make a logfile.
  ustring logfile;
  logfile = gw_build_filename (directories_get_temp(), "shell.log");
  // Start the process in the background.
  ustring command (commandline);
  // Route output to the logfile.
  command.append (" > ");
  command.append (logfile);
  command.append (" 2>&1");
  command.append (" &");
  if (system (command.c_str()) >= 0) {
    // Indicate zero for no error.
    returnvalue = 0;
    // We show a moving fraction.
    double fraction = 0;
    // Wait for the shell process to finish.
    while (program_is_running (programname)) {
      fraction += 0.02;
      if (fraction >= 1)
        fraction = 0;
      progress->set_fraction (fraction);
      // Check the size of the logfile is stil within limits.
      struct stat statbuf;
      stat (logfile.c_str(), &statbuf);
      if (statbuf.st_size > max_logfile_size)
        size_exceeded = true;
      // Check whether cancel.
      if (progress->cancel || size_exceeded) {
        // Kill formatter.
        ustring command;
        command = "killall -HUP ";
        command.append (programname);
        system (command.c_str());
        returnvalue = -1;
        break;
      }
      g_usleep (1000000);
    }
  }
  // Give message if logsize was exceeded.
  if (size_exceeded)
    gtkw_dialog_error (NULL, "Process produced a logfile which became too large. Interrupted");
  // Read the logfile made by the shell command.
  if (output_to_logfile) {
    ReadText rt (logfile, true, false);
    for (unsigned int i = 0; i < rt.lines.size(); i++) {
      // Add to program's logfile.
      gw_message (rt.lines[i]);
    }
  }
  // Finish.
  progress->finish();
  return returnvalue;  
}
